@@ -32,7 +32,7 @@
  * 
  *
  * CONTRIBUTORS
- *  Stef Walter <stef@memberwebs.com>
+ *  Stefan Walter <stef@memberwebs.com>
  *  Andreas Steinmetz <ast@domdv.de>
  *  Rubio Vaughan <rubio@passim.net>
  *  Olivier Beyssac <ob@r14.freenix.org>
@@ -98,6 +98,7 @@ spthread_t;
 
 #define SMTP_DATA           "DATA" CRLF
 #define SMTP_NOOP           "NOOP" CRLF
+#define SMTP_RSET           "RSET" CRLF
 #define SMTP_XCLIENT        "XCLIENT ADDR=%s" CRLF
 #define SMTP_BANNER         "220 smtp.passthru" CRLF
 #define SMTP_HELO_RSP       "250 smtp.passthru" CRLF
@@ -656,6 +657,12 @@ static void cleanup_context(spctx_t* ctx)
         ctx->xforwardaddr = NULL;
     }
     
+    if(ctx->xforwardhelo)
+    {
+    	free(ctx->xforwardhelo);
+    	ctx->xforwardhelo = NULL;
+    }
+    
     ctx->logline[0] = 0;
 }
 
@@ -1131,6 +1138,14 @@ static int smtp_passthru(spctx_t* ctx)
                         if(ctx->xforwardaddr)
                             strcpy(ctx->xforwardaddr, t);
                     }
+                    
+                    if((t = parse_xforward (C_LINE + KL(XFORWARD_CMD), "HELO"))) 
+                    {
+                        ctx->xforwardhelo = (char*)reallocf(ctx->xforwardhelo, strlen(t) + 1);
+                        if(ctx->xforwardhelo)
+                            strcpy(ctx->xforwardhelo, t);
+                    }
+                    
                 }
                 
                 /* RSET */
@@ -1710,6 +1725,11 @@ int sp_fail_data(spctx_t* ctx, const char* smtp_status)
     
     if(spio_write_data(ctx, &(ctx->client), smtp_status) == -1)
         return -1;
+
+     /* Tell the server to forget about the current message */
+     if(spio_write_data(ctx, &(ctx->server), SMTP_RSET) == -1 || 
+        read_server_response(ctx) == -1)
+         return -1;
         
     return 0;
 }
@@ -1771,6 +1791,9 @@ void sp_setup_forked(spctx_t* ctx, int file)
     
     if(ctx->xforwardaddr)
         setenv("REMOTE", ctx->xforwardaddr, 1);
+        
+    if(ctx->xforwardhelo)
+        setenv("REMOTE_HELO", ctx->xforwardhelo, 1);
     
     if(spio_valid(&(ctx->server)))
         setenv("SERVER", ctx->server.peername, 1);
