@@ -709,7 +709,9 @@ static int virus_action(clctx_t* ctx, const char* virus)
     if(g_clstate.quarantine)
     {
         strlcpy(qfilename, g_clstate.directory, MAXPATHLEN);
-        strlcat(qfilename, "/virus.", MAXPATHLEN);
+	if (qfilename[strlen(qfilename) - 1] != '/')
+		strlcat(qfilename, "/", MAXPATHLEN);
+        strlcat(qfilename, "virus.", MAXPATHLEN);
     
         /* Points to null terminator */
         t = qfilename + strlen(qfilename);
